@@ -1,17 +1,34 @@
 #ifndef _DEVICES_H_
 #define _DEVICES_H_
 
-#include "atari.h"	/* for UWORD */
-
-/* following is for atari.c */
+#include <stdio.h> /* FILENAME_MAX */
+#include "atari.h" /* UWORD */
 
 void Device_Initialise(int *argc, char *argv[]);
 int Device_PatchOS(void);
 void Device_Frame(void);
 void Device_UpdatePatches(void);
-void Device_HHINIT(void);
 
-/* following is for ports which want to add their specific devices */
+UWORD Device_SkipDeviceName(void);
+
+extern int enable_h_patch;
+extern int enable_p_patch;
+extern int enable_r_patch;
+
+extern char atari_h_dir[4][FILENAME_MAX];
+extern int h_read_only;
+
+#define DEFAULT_H_PATH  "H1:>DOS;>DOS"
+extern char h_exe_path[FILENAME_MAX];
+
+char h_current_dir[4][FILENAME_MAX];
+
+int Device_H_CountOpen(void);
+void Device_H_CloseAll(void);
+
+extern char print_command[256];
+
+int Device_SetPrintCommand(const char *command);
 
 #define	ICHIDZ	0x0020
 #define	ICDNOZ	0x0021
@@ -44,7 +61,6 @@ void Device_HHINIT(void);
 #define	ICAX5	0x000e
 #define	ICAX6	0x000f
 
-
 #define DEVICE_TABLE_OPEN	0
 #define DEVICE_TABLE_CLOS	2
 #define DEVICE_TABLE_READ	4
