@@ -2,7 +2,7 @@
  * atari_vga.c - DOS VGA mode specific port code
  *
  * Copyright (c) 1996 Ivo van Poorten
- * Copyright (c) 1996-2003 Atari800 development team (see DOC/CREDITS)
+ * Copyright (c) 1996-2005 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -37,12 +37,10 @@
 #include "input.h"
 #include "ui.h"         /* for ui_is_active */
 #include "log.h"
-#include "dos/sound_dos.h"
+#include "sound.h"
 #include "monitor.h"
 #include "pcjoy.h"
-#include "ataripcx.h"
-#include "rt-config.h"	/* for refresh_rate */
-#include "diskled.h"	/* for led_status */
+#include "screen.h"
 
 #include "dos/vga_gfx.h"
 
@@ -218,24 +216,9 @@ static void update_leds(void)
 	outportb(0x60,0xed);
 	asm("nop; nop");
 	outportb(0x60,	(PC_keyboard ? 0 : 2)
-#ifdef SHOW_DISK_LED
-			|(led_status ? 4 : 0)
-#endif
 			|(joy_keyboard ? 1 : 0));
 }
 
-static void update_disk_led()
-{
-#ifdef SHOW_DISK_LED
-	static int last_led_status = 0;
-
-	if (led_status != last_led_status) {
-		update_leds();
-		last_led_status = led_status;
-	}
-#endif
-}
-
 /* -------------------------------------------------------------------------- */
 /* KEYBOARD HANDLER                                                           */
 /* -------------------------------------------------------------------------- */
@@ -447,7 +430,7 @@ void key_delete(void)
 /* VGA HANDLING                                                               */
 /* -------------------------------------------------------------------------- */
 
-void SetupVgaEnvironment()
+void SetupVgaEnvironment(void)
 {
         int a, i;
         union REGS rg;
@@ -460,7 +443,7 @@ void SetupVgaEnvironment()
           use_vesa=VESA_open(vesa_mode,vesa_memptr,vesa_memsize,&vesa_linear,&vesa_selector);
         }
         if (!use_vesa) /*if '-novesa' specified or VESA_open failed */
-          switch(video_mode)
+          switch (video_mode)
           {
             case 0:
               rg.x.ax = 0x0013;
@@ -517,7 +500,7 @@ void SetupVgaEnvironment()
         key_init(); /*initialize keyboard handler*/
 }
 
-void ShutdownVgaEnvironment()
+void ShutdownVgaEnvironment(void)
 {
         union REGS rg;
 
@@ -533,8 +516,9 @@ void ShutdownVgaEnvironment()
    vga_started=FALSE;
 }
 
-void Atari_DisplayScreen(UBYTE * ascreen)
+void Atari_DisplayScreen(void)
 {
+        UBYTE *ascreen = (UBYTE *) atari_screen;
         static int lace = 0;
         unsigned long vga_ptr;
 
@@ -560,7 +544,7 @@ void Atari_DisplayScreen(UBYTE * ascreen)
 
         if (use_vesa)
             /*draw screen using VESA2*/
-            switch(video_mode)
+            switch (video_mode)
             {
               case 0:
                 VESA_blit(ascreen+first_col + first_lno*ATARI_WIDTH,
@@ -581,7 +565,7 @@ void Atari_DisplayScreen(UBYTE * ascreen)
             }
         else
             /*draw screen using vga or x-mode*/
-            switch(video_mode)
+            switch (video_mode)
             {
               case 0:
                 for (ypos = 0; ypos < 200; ypos += ypos_inc) {
@@ -702,7 +686,7 @@ void Atari_Initialise(int *argc, char *argv[])
         /*find number of VESA2 video mode*/
         if (use_vesa)
         {
-          switch(video_mode)
+          switch (video_mode)
           {
             case 0:
               use_vesa=VESA_getmode(320,200,&vesa_mode,&vesa_memptr,&vesa_linelenght,&vesa_memsize);
@@ -870,6 +854,8 @@ int Atari_Exit(int run_monitor)
 
         key_delete();                           /* enable keyboard in monitor */
 
+        Aflushlog();
+
         if (run_monitor) {
 #ifdef SOUND
                 Sound_Pause();
@@ -890,8 +876,6 @@ int Atari_Exit(int run_monitor)
         Sound_Exit();
 #endif
 
-        Aflushlog();
-
         return 0;
 }
 
@@ -960,49 +944,49 @@ int Atari_Keyboard(void)
                         keycode = AKEY_EXIT;
                         break;
                 case 0x02:
-                        keycode = 0x3f;
+                        keycode = AKEY_5200_1;
                         break;
                 case 0x03:
-                        keycode = 0x3d;
+                        keycode = AKEY_5200_2;
                         break;
                 case 0x04:
-                        keycode = 0x3b;
+                        keycode = AKEY_5200_3;
                         break;
                 case 0x0D:
-                        keycode = 0x23;         /* = = * */
+                        keycode = AKEY_5200_HASH;         /* = = * */
                         break;
                 case 0x05:
-                        keycode = 0x37;
+                        keycode = AKEY_5200_4;
                         break;
                 case 0x06:
-                        keycode = 0x35;
+                        keycode = AKEY_5200_5;
                         break;
                 case 0x07:
-                        keycode = 0x33;
+                        keycode = AKEY_5200_6;
                         break;
                 case 0x08:
-                        keycode = 0x2f;
+                        keycode = AKEY_5200_7;
                         break;
                 case 0x09:
-                        keycode = 0x2d;
+                        keycode = AKEY_5200_8;
                         break;
                 case 0x0C:
-                        keycode = 0x27;         /* - = * */
+                        keycode = AKEY_5200_ASTERISK;         /* - = * */
                         break;
                 case 0x0a:
-                        keycode = 0x2b;
+                        keycode = AKEY_5200_9;
                         break;
                 case 0x0b:
-                        keycode = 0x25;
+                        keycode = AKEY_5200_0;
                         break;
                 case 0x3e:                              /* 1f : */
-                        keycode = 0x39;         /* start */
+                        keycode = AKEY_5200_START;         /* start */
                         break;
                 case 0x19:
-                        keycode = 0x31;         /* pause */
+                        keycode = AKEY_5200_PAUSE;         /* pause */
                         break;
                 case 0x13:
-                        keycode = 0x29;         /* reset */
+                        keycode = AKEY_5200_RESET;         /* reset */
                         break;
                 case 0x42:                              /* F8 */
                         if (!norepkey) {
@@ -1041,12 +1025,7 @@ int Atari_Keyboard(void)
                         keycode = AKEY_UI;
                 break;
         case 0xCF:                                      /* gray END*/
-                if (!norepkey) {
-                        keycode = AKEY_BREAK;
-                        norepkey = TRUE;
-                }
-                else
-                        keycode = AKEY_NONE;
+                keycode = AKEY_BREAK;
                 break;
         case 0x44:                                      /* F10 */
                 if (!norepkey)
@@ -1061,10 +1040,7 @@ int Atari_Keyboard(void)
                 keycode = key_shift ? AKEY_COLDSTART : AKEY_WARMSTART;
                 break;
         case 0x40:                                      /* F6 */
-                if (machine_type != MACHINE_XLXE)
-                        keycode = AKEY_PIL;
-                else
-                        keycode = AKEY_HELP;
+                keycode = AKEY_HELP;
                 break;
         case 0x42:                                      /* F8 */
                 if (!norepkey && !ui_is_active) {
@@ -1079,11 +1055,11 @@ int Atari_Keyboard(void)
                 break;
 		case 0x57:					/* F11 */
 		 		keycode = AKEY_NONE;
-				for(i = 0; i < 4; i++) {
-		 		  if (++joy_autofire[i] > 2)
-		 		    joy_autofire[i] = 0;
+				for (i = 0; i < 4; i++) {
+					if (++joy_autofire[i] > 2)
+						joy_autofire[i] = 0;
 		 		}
-		 		raw_key = 0;	/* aviod continuous change */
+		 		raw_key = 0;	/* avoid continuous change */
 		 		break;
         case 0x01:
                 keycode |= AKEY_ESCAPE;
@@ -1371,7 +1347,7 @@ int Atari_Keyboard(void)
         case 0x2e:
 				if (alt_key)
 				{
-					keycode = AKEY_UI;			
+					keycode = AKEY_UI;
 					alt_function = MENU_CARTRIDGE;			/* ALT+C .. Cartridge management */
 				}
 				else
@@ -1414,7 +1390,7 @@ int Atari_Keyboard(void)
 
 
         case 0xc7:                                      /* HOME key */
-                keycode |= 118;                 /* clear screen */
+                keycode |= AKEY_CLEAR;
                 break;
         case 0xd2:                                      /* INSERT key */
                 if (key_shift)
@@ -1475,43 +1451,8 @@ int main(int argc, char **argv)
 		return 3;
 
 	/* main loop */
-	while (TRUE) {
-		int refresh_counter = 0;
-		int keycode = Atari_Keyboard();
-
-		switch (keycode) {
-		case AKEY_COLDSTART:
-			Coldstart();
-			break;
-		case AKEY_WARMSTART:
-			Warmstart();
-			break;
-		case AKEY_EXIT:
-			Atari800_Exit(FALSE);
-			exit(1);
-		case AKEY_UI:
-#ifdef SOUND
-			Sound_Pause();
-#endif
-			ui((UBYTE *)atari_screen);
-#ifdef SOUND
-			Sound_Continue();
-#endif
-			break;
-		case AKEY_SCREENSHOT:
-			Save_PCX_file(FALSE, Find_PCX_name());
-			break;
-		case AKEY_SCREENSHOT_INTERLACE:
-			Save_PCX_file(TRUE, Find_PCX_name());
-			break;
-		case AKEY_BREAK:
-			key_break = 1;
-			break;
-		default:
-			key_break = 0;
-			key_code = keycode;
-			break;
-		}
+	for (;;) {
+		key_code = Atari_Keyboard();
 
 		if (mouse_mode != MOUSE_OFF) {
 			union REGS rg;
@@ -1524,24 +1465,8 @@ int main(int argc, char **argv)
 			mouse_buttons = rg.x.bx;
 		}
 
-		if (++refresh_counter == refresh_rate) {
-			Atari800_Frame(EMULATE_FULL);
-#ifndef DONT_SYNC_WITH_HOST
-			atari_sync(); /* here seems to be the best place to sync */
-#endif
-			update_disk_led();
-			Atari_DisplayScreen((UBYTE *) atari_screen);
-			refresh_counter = 0;
-		}
-		else {
-#ifdef VERY_SLOW
-			Atari800_Frame(EMULATE_BASIC);
-#else	/* VERY_SLOW */
-			Atari800_Frame(EMULATE_NO_SCREEN);
-#ifndef DONT_SYNC_WITH_HOST
-			atari_sync();
-#endif
-#endif	/* VERY_SLOW */
-		}
+		Atari800_Frame();
+		if (display_screen)
+			Atari_DisplayScreen();
 	}
 }
