@@ -2,7 +2,7 @@
  * log.c - A logging facility for debugging
  *
  * Copyright (C) 1995-1998 David Firth
- * Copyright (C) 1998-2003 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 1998-2005 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -22,10 +22,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "config.h"
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
-#include "config.h"
+
 #include "log.h"
 
 #ifdef MACOSX
@@ -34,10 +35,11 @@
 #  define PRINT(a) printf("%s", a)
 #endif
 
-#define MAX_LOG_SIZE		8192
-char memory_log[MAX_LOG_SIZE]="";
+#ifdef BUFFERED_LOG
+char memory_log[MAX_LOG_SIZE] = "";
+#endif
 
-void Aprint(char *format, ... )
+void Aprint(char *format, ...)
 {
 	va_list args;
 	char buffer[8192];
@@ -46,10 +48,18 @@ void Aprint(char *format, ... )
 #endif
 
 	va_start(args, format);
-	vsnprintf(buffer, sizeof(buffer)-1 /* -1 for the strcat(\n) */, format, args);
+#ifdef HAVE_VSNPRINTF
+	vsnprintf(buffer, sizeof(buffer) - 2 /* -2 for the strcat() */, format, args);
+#else
+	vsprintf(buffer, format, args);
+#endif
 	va_end(args);
 
+#ifdef __PLUS
+	strcat(buffer, "\r\n");
+#else
 	strcat(buffer, "\n");
+#endif
 
 #ifdef BUFFERED_LOG
 	buflen = strlen(buffer);
@@ -67,31 +77,8 @@ void Aflushlog(void)
 {
 #ifdef BUFFERED_LOG
 	if (*memory_log) {
-		printf(memory_log);
+		PRINT(memory_log);
 		*memory_log = 0;
 	}
 #endif
 }
-
-/*
-$Log: log.c,v $
-Revision 1.7  2003/11/13 13:09:02  joy
-buffer overflow fixed
-
-Revision 1.6  2003/10/25 18:40:54  joy
-various little updates for better MacOSX support
-
-Revision 1.5  2003/02/24 09:33:01  joy
-header cleanup
-
-Revision 1.4  2002/04/07 05:44:47  vasyl
-Log buffer is completely hidden inside C file (no extern in header). This allows
-log-less ports to save extra 8K.
-
-Revision 1.3  2001/12/04 14:17:52  joy
-Aflushlog() should be always available though it does nothing when BUFFERED_LOG is undefined
-
-Revision 1.2  2001/03/18 06:34:58  knik
-WIN32 conditionals removed
-
-*/
