@@ -1,5 +1,5 @@
-#ifndef __PIA__
-#define __PIA__
+#ifndef _PIA_H_
+#define _PIA_H_
 
 #include "atari.h"
 
@@ -12,7 +12,8 @@ extern UBYTE PACTL;
 extern UBYTE PBCTL;
 extern UBYTE PORTA;
 extern UBYTE PORTB;
-
+extern UBYTE PORTA_mask;
+extern UBYTE PORTB_mask;
 extern UBYTE PORT_input[2];
 
 extern int xe_bank;
@@ -22,7 +23,8 @@ extern UBYTE atari_basic[8192];
 extern UBYTE atari_os[16384];
 
 void PIA_Initialise(int *argc, char *argv[]);
+void PIA_Reset(void);
 UBYTE PIA_GetByte(UWORD addr);
 void PIA_PutByte(UWORD addr, UBYTE byte);
 
-#endif
+#endif /* _PIA_H_ */
