@@ -98,22 +98,32 @@ extern "C" {
 #define SAMP_MID 128
 #endif
 
-// init flags
+/* init flags */
 #define SND_BIT16	1
 #define SND_STEREO	2
 
-extern void (* Pokey_process)(void * sndbuffer, unsigned int sndn);
-extern void (* Update_pokey_sound)(uint16 addr, uint8 val, uint8 /*chip*/, uint8 gain);
-extern void (* Update_serio_sound)(int out, UBYTE data);
-extern void (* Update_consol_sound)(int set);
-extern void (* Update_vol_only_sound)(void);
+extern int32 snd_playback_freq;
+extern uint8 snd_num_pokeys;
 
-int Pokey_sound_init(uint32 freq17,
-			     uint16 playback_freq,
-			     uint8 num_pokeys,
-			     unsigned int flags
-			    );
-int Pokey_DoInit();
+extern int enable_new_pokey;
+extern int stereo_enabled;
+extern int serio_sound_enabled;
+extern int console_sound_enabled;
+
+extern void (*Pokey_process_ptr)(void *sndbuffer, unsigned int sndn);
+extern void (*Update_pokey_sound)(uint16 addr, uint8 val, uint8 /*chip*/, uint8 gain);
+extern void (*Update_serio_sound)(int out, UBYTE data);
+extern void (*Update_consol_sound)(int set);
+extern void (*Update_vol_only_sound)(void);
+
+int Pokey_sound_init(uint32 freq17, uint16 playback_freq, uint8 num_pokeys,
+                     unsigned int flags
+#ifdef __PLUS
+                     , int clear_regs
+#endif
+                     );
+void Pokey_process(void *sndbuffer, unsigned int sndn);
+int Pokey_DoInit(void);
 void Pokey_set_mzquality(int quality);
 
 #ifdef __cplusplus
