@@ -485,6 +485,7 @@ int SelectCartType(int k)
 		MENU_ACTION(40, "Blizzard 16 KB cartridge"),
 		MENU_ACTION(41, "Atarimax 128 KB Flash cartridge"),
 		MENU_ACTION(42, "Atarimax 1 MB Flash cartridge"),
+		MENU_ACTION(43, "SpartaDOS X 128 KB cartridge"),
 		MENU_END
 	};
 
@@ -1113,6 +1114,7 @@ static void DisplaySettings(void)
 	static char refresh_status[16];
 	static tMenuItem menu_array[] = {
 		MENU_SUBMENU_SUFFIX(0, "Artifacting mode:", NULL),
+		MENU_CHECK(11, "Enable new artifacting:"),
 		MENU_SUBMENU_SUFFIX(1, "Current refresh rate:", refresh_status),
 		MENU_CHECK(2, "Accurate skipped frames:"),
 		MENU_CHECK(3, "Show percents of Atari speed:"),
@@ -1136,6 +1138,7 @@ static void DisplaySettings(void)
 	int option2;
 	for (;;) {
 		FindMenuItem(menu_array, 0)->suffix = artif_menu_array[global_artif_mode].item;
+		SetItemChecked(menu_array, 11, artif_new);
 		sprintf(refresh_status, "1:%-2d", refresh_rate);
 		SetItemChecked(menu_array, 2, sprite_collisions_in_skipped_frames);
 		SetItemChecked(menu_array, 3, show_atari_speed);
@@ -1158,6 +1161,10 @@ static void DisplaySettings(void)
 				ANTIC_UpdateArtifacting();
 			}
 			break;
+		case 11:
+			artif_new = !artif_new;
+			ANTIC_UpdateArtifacting();
+			break;
 		case 1:
 			refresh_rate = ui_driver->fSelectInt(refresh_rate, 1, 99);
 			break;
