@@ -2,7 +2,7 @@
  * main.c - Win32 port specific code
  *
  * Copyright (C) 2000 Krzysztof Nikiel
- * Copyright (C) 2000-2005 Atari800 development team (see DOC/CREDITS)
+ * Copyright (C) 2000-2006 Atari800 development team (see DOC/CREDITS)
  *
  * This file is part of the Atari800 emulator project which emulates
  * the Atari 400, 800, 800XL, 130XE, and 5200 8-bit computers.
@@ -60,7 +60,7 @@ void exit(int code)
 }
 #endif
 
-static long FAR PASCAL WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
+LRESULT CALLBACK Atari_WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
 	switch (message) {
 	case WM_ACTIVATEAPP:
@@ -74,14 +74,20 @@ static long FAR PASCAL WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM
 		break;
 	case WM_LBUTTONDOWN:
 	case WM_LBUTTONUP:
+	case WM_MBUTTONDOWN:
+	case WM_MBUTTONUP:
 	case WM_RBUTTONDOWN:
 	case WM_RBUTTONUP:
 		mouse_buttons = ((wParam & MK_LBUTTON) ? 1 : 0)
-		              | ((wParam & MK_RBUTTON) ? 2 : 0);
+		              | ((wParam & MK_RBUTTON) ? 2 : 0)
+		              | ((wParam & MK_MBUTTON) ? 4 : 0);
 		break;
 	case WM_SETCURSOR:
-		SetCursor(NULL);
-		return TRUE;
+		if (!windowed) {
+			SetCursor(NULL);
+			return TRUE;
+		}
+		break;
 	case WM_CREATE:
 		break;
 	case WM_CLOSE:
@@ -99,46 +105,15 @@ static long FAR PASCAL WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM
 	return DefWindowProc(hWnd, message, wParam, lParam);
 }
 
-static BOOL initwin(HINSTANCE hInstance, int nCmdShow)
-{
-	WNDCLASS wc;
-
-	wc.style = CS_HREDRAW | CS_VREDRAW;
-	wc.lpfnWndProc = WindowProc;
-	wc.cbClsExtra = 0;
-	wc.cbWndExtra = 0;
-	wc.hInstance = hInstance;
-	wc.hIcon = LoadIcon(hInstance, IDI_APPLICATION);
-	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
-	wc.hbrBackground = GetStockObject(BLACK_BRUSH);
-	wc.lpszMenuName = myname;
-	wc.lpszClassName = myname;
-	RegisterClass(&wc);
-
-	hWndMain = CreateWindowEx(
-		0, myname, myname, WS_POPUP, 0, 0,
-		GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN),
-		NULL, NULL, hInstance, NULL);
-
-	if (!hWndMain)
-		return 1;
-	return 0;
-}
-
 #define MOUSE_CENTER_X  100
 #define MOUSE_CENTER_Y  100
 
 int main(int argc, char *argv[])
 {
-	STARTUPINFO si;
 	MSG msg;
 	POINT mouse;
 
 	myInstance = GetModuleHandle(NULL);
-	si.dwFlags = 0;
-	GetStartupInfo(&si);
-	if (initwin(myInstance, si.dwFlags & STARTF_USESHOWWINDOW ? si.wShowWindow : SW_SHOWDEFAULT))
-		return 1;
 
 	/* initialise Atari800 core */
 	if (!Atari800_Initialise(&argc, argv))
@@ -165,7 +140,7 @@ int main(int argc, char *argv[])
 		GetCursorPos(&mouse);
 		mouse_delta_x = mouse.x - MOUSE_CENTER_X;
 		mouse_delta_y = mouse.y - MOUSE_CENTER_Y;
-		if (mouse_delta_x | mouse_delta_y)
+		if (!windowed && (mouse_delta_x | mouse_delta_y))
 			SetCursorPos(MOUSE_CENTER_X, MOUSE_CENTER_Y);
 
 		Atari800_Frame();
